
stat
	parent_type = /obj		// make this an atomic object so it is clickable and has a suffix in the stat panel
	var
		mob/owner
		statname = ""		// text string of what value you are increasing
		value = 0			// the value of the stat
		stat_inc = 1		// how many statpoints it consumes to add 1 to the value


	New(location, _owner, _stat)
		// set the owner, and stat variable when created
		src.owner = _owner
		src.statname = _stat

		spawn()
			Update()

	proc
		Update()	// call this every time your stats have changed (strength, defense, etc...)
			value = owner.vars[statname]
			name = "[value]"
			suffix = "[stat_inc] points to increase"
			//	Update the value, name, and suffix


		StatIncrease(mob/M)		// use this when needed to increase the value by using stat points
			if(src.owner == M)			// check if they own the stat object
				if(M.statpoints >= stat_inc)	// check if they have enough stat points
					M.statpoints -= stat_inc
					Increase()
				else
					M << "You do not have enough stat points to increase your [stat_convert(statname)]."

		Increase()	// use this to just increase the stat value by one if not neccessarily needing to use statpoints
			value++
			owner.vars[statname] = value
			if(!(owner.vars["[statname]lv"] % value)) // if the stat is a multiple of the value
				stat_inc++	// increase the requirement to add the stat

			Update()
			owner.SkillUpdate()
			// increase the stat_inc for every statlv it hits, then update


	Click()
		StatIncrease(usr)	// StatIncrease() gets called when clicked



skill	// some example skills used in the demo
	parent_type = /obj

	attack
		name = "Attack"
	fireball
		name = "Fireball"
	quickslash
		name = "Quick Slash"
	bigslash
		name = "Big Slash"
	ultimateattack
		name = "Ultimate Attack"


mob
	var
		statpoints = 20	// the credit used to increase stats

		str = 1
		def = 1
		int = 1
		agi = 1

		strlv = 5		// for every 5 stats (by default), the stat point requirement increases by one
		deflv = 5
		intlv = 5
		agilv = 5

		list
			statobjs	// list of stat objects (/statobj)
			skills = newlist(/skill/attack)			// skills
			potential_skills = list(/skill/fireball			= list("int"=7),
									/skill/quickslash		= list("agi"=5),
									/skill/bigslash			= list("str"=10),
									/skill/ultimateattack	= list("str"=10,"def"=10,"int"=10,"agi"=10))

			// skills that can be learned; an associative list where the element is
			// the path type of the skill, and the association is a list of the required
			// stats: list(abbreviated_stat_name = required_stat_to_learn_skill)

	Login()
		AddStats()			// add the stat objects for the player
		StatDistribute()	// start the stat distribution when they log in
		world << "[src] has joined the world."
		..()

	Stat()
		if(statpanel("Statistics"))
			for(var/stat/s in statobjs)
				stat(stat_convert(s.statname), s)
			stat("Stat points: ", statpoints)
			stat(null)
			stat("Skills: ", null)
			stat(skills)

	verb
		add_statpoints(n as num)		// the test verb to add stat points manually
			src.statpoints += round(max(1, n))


	proc
		StatDistribute()
			var/credits = 10
			var/list/stats = list("Strength"="str", "Defense"="def",
								"Intelligence"="int", "Agility"="agi")

			while(credits)	// keep on looping until there are no more credits
				var/list/L = formatted_stat_list(stats, src)
				// create a formatted list to let the user see how much of a stat they have
				var/i = input(src, "Customize stats.  You have [credits] credits left.", "Stats") \
					as null|anything in L
				if(!i) break	// let them end whenever they want, if they want to miss out of free points :P
				credits--		// otherwise remove one credit
				for(var/stat/s in statobjs)
					if(s.statname == L[i])
						s.Increase()	// tell the statobj to increase the value that you inputted


		SkillUpdate()
			var/list/learned = list()	// list of learned skills in the level-up

			for(var/i in potential_skills) // for every path type the player can potentially learn
				var/amount = 0	// number indicating how many stat requirements the player has achived
				for(var/j in potential_skills[i])	// check every requirement in the association
					var/x = potential_skills[i]
					if(src.vars[j] >= x[j])	// if the requirement is good enough, add the amount by 1
						amount++
				if(amount == length(potential_skills[i]))	// if they have filled all the requirements
					learned += i	// let them learn the skill

			for(var/skill in learned)
				if(!(locate(skill) in skills))	// check if they don't already know the skill
					var/skill/newskill = new skill
					skills += newskill	// add all new skils
					src << "You have learned the skill [newskill]"


		// Call AddStats() whenever you need to add the statpanels or stats.
		// This must be called before StatDistribute() because the stat objects
		// are relied on them.
		AddStats()
			statobjs = list()
			var/list/slist = list("str","def","int","agi")
			for(var/i in slist)
				src.statobjs += new /stat (null, src, i)

proc

	// returns the new formatted list as an associative list: list("stat name and value" = "actual stat")
	formatted_stat_list(list/L, mob/M)
		var/list/new_list = list()
		for(var/i in L)
			new_list["[i] -- [M.vars[L[i]]]"] = L[i]
			// make an item that represents the stat and the actual amount, and
			// set the association to the stat name that is going to be increased
		return new_list


	stat_convert(t)	// a function that returns a converted form of wording (abbreviated to full)
		var/list/A = list("Strength","Defense","Intelligence","Agility")
		var/list/B = list("str","def","int","agi")

		// a handy way to use parallel lists to convert stuff back and forth
		// if the text is found in the A list, return the item in the same
		// placement in the B list
		return (t in A) ? B[A.Find(t)] : A[B.Find(t)]

		// The ? operator is used for readability and compactness for some people,
		// but the operator may obfusicate the reader of the code when mishandled

		// However, the above line is equivalent to the following:

	/*
		if(t in A)	// if t is found in A
			return B[A.Find(t)] // return the value in the same placement of A from the B list
		else
			return A[B.Find(t)] // return the value in the same placement of B from the A list
	 */